package javax.management.openmbean;

import java.io.IOException;
import sun.misc.SharedSecrets;
import java.io.ObjectInputStream;
import java.util.Collections;
import java.util.Iterator;
import com.sun.jmx.mbeanserver.Util;
import java.util.Collection;
import java.util.Set;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.LinkedHashMap;
import java.util.HashMap;
import java.security.AccessController;
import com.sun.jmx.mbeanserver.GetPropertyAction;
import java.io.Serializable;
import java.util.Map;

public class TabularDataSupport implements TabularData, Map<Object, Object>, Cloneable, Serializable
{
    static final long serialVersionUID = 5720150593236309827L;
    private Map<Object, CompositeData> dataMap;
    private final TabularType tabularType;
    private transient String[] indexNamesArray;
    
    public TabularDataSupport(TabularType tabularType) {
        this(tabularType, 16, 0.75f);
    }
    
    public TabularDataSupport(TabularType tabularType, int n, float n2) {
        if (tabularType == null) {
            throw new IllegalArgumentException("Argument tabularType cannot be null.");
        }
        this.tabularType = tabularType;
        List<String> indexNames = tabularType.getIndexNames();
        this.indexNamesArray = (String[])indexNames.toArray(new String[indexNames.size()]);
        this.dataMap = ("true".equalsIgnoreCase((String)AccessController.doPrivileged(new GetPropertyAction("jmx.tabular.data.hash.map"))) ? new HashMap(n, n2) : new LinkedHashMap(n, n2));
        return;
    }
    
    @Override
    public TabularType getTabularType() {
        return this.tabularType;
    }
    
    @Override
    public Object[] calculateIndex(CompositeData compositeData) {
        this.checkValueType(compositeData);
        return this.internalCalculateIndex(compositeData).toArray();
    }
    
    @Override
    public boolean containsKey(Object o) {
        Object[] array;
        try {
            array = (Object[])(Object[])o;
        }
        catch (ClassCastException ex) {
            return false;
        }
        return this.containsKey(array);
    }
    
    @Override
    public boolean containsKey(Object[] a) {
        return a != null && this.dataMap.containsKey(Arrays.asList(a));
    }
    
    @Override
    public boolean containsValue(CompositeData compositeData) {
        return this.dataMap.containsValue(compositeData);
    }
    
    @Override
    public boolean containsValue(Object o) {
        return this.dataMap.containsValue(o);
    }
    
    @Override
    public Object get(Object o) {
        return this.get((Object[])(Object[])o);
    }
    
    @Override
    public CompositeData get(Object[] a) {
        this.checkKeyType(a);
        return (CompositeData)this.dataMap.get(Arrays.asList(a));
    }
    
    @Override
    public Object put(Object o, Object o2) {
        this.internalPut((CompositeData)o2);
        return o2;
    }
    
    @Override
    public void put(CompositeData compositeData) {
        this.internalPut(compositeData);
    }
    
    private CompositeData internalPut(CompositeData compositeData) {
        return (CompositeData)this.dataMap.put(this.checkValueAndIndex(compositeData), compositeData);
    }
    
    @Override
    public Object remove(Object o) {
        return this.remove((Object[])(Object[])o);
    }
    
    @Override
    public CompositeData remove(Object[] a) {
        this.checkKeyType(a);
        return (CompositeData)this.dataMap.remove(Arrays.asList(a));
    }
    
    @Override
    public void putAll(Map<?, ?> map) {
        if (map == null || map.size() == 0) {
            return;
        }
        CompositeData[] array;
        try {
            array = (CompositeData[])map.values().toArray(new CompositeData[map.size()]);
        }
        catch (ArrayStoreException ex) {
            throw new ClassCastException("Map argument t contains values which are not instances of <tt>CompositeData</tt>");
        }
        this.putAll(array);
        return;
    }
    
    @Override
    public void putAll(CompositeData[] array) {
        if (array == null || array.length == 0) {
            return;
        }
        ArrayList<List> list = new ArrayList(array.length + 1);
        int i = 0;
        while (i < array.length) {
            List<Object> checkValueAndIndex = this.checkValueAndIndex(array[i]);
            if (list.contains(checkValueAndIndex)) {
                throw new KeyAlreadyExistsException("Argument elements values[" + i + "] and values[" + list.indexOf(checkValueAndIndex) + "] have the same indexes, calculated according to this TabularData instance's tabularType.");
            }
            list.add(checkValueAndIndex);
            ++i;
        }
        for (int j = 0; j < array.length; ++j) {
            this.dataMap.put(list.get(j), array[j]);
        }
        return;
    }
    
    @Override
    public void clear() {
        this.dataMap.clear();
    }
    
    @Override
    public int size() {
        return this.dataMap.size();
    }
    
    @Override
    public boolean isEmpty() {
        return this.size() == 0;
    }
    
    @Override
    public Set<Object> keySet() {
        return this.dataMap.keySet();
    }
    
    @Override
    public Collection<Object> values() {
        return (Collection)Util.cast(this.dataMap.values());
    }
    
    @Override
    public Set<Map.Entry<Object, Object>> entrySet() {
        return (Set)Util.cast(this.dataMap.entrySet());
    }
    
    public Object clone() {
        try {
            TabularDataSupport tabularDataSupport = (TabularDataSupport)super.clone();
            tabularDataSupport.dataMap = new HashMap(tabularDataSupport.dataMap);
            return tabularDataSupport;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause.toString(), cause);
        }
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == null) {
            return false;
        }
        TabularData tabularData;
        try {
            tabularData = (TabularData)o;
        }
        catch (ClassCastException ex) {
            return false;
        }
        if (!this.getTabularType().equals(tabularData.getTabularType())) {
            return false;
        }
        if (this.size() != tabularData.size()) {
            return false;
        }
        Iterator<CompositeData> iterator = this.dataMap.values().iterator();
        while (iterator.hasNext()) {
            if (!tabularData.containsValue((CompositeData)iterator.next())) {
                return false;
            }
            continue;
        }
        return true;
    }
    
    @Override
    public int hashCode() {
        int n = 0 + this.tabularType.hashCode();
        Iterator<Object> iterator = this.values().iterator();
        while (iterator.hasNext()) {
            n += iterator.next().hashCode();
        }
        return n;
    }
    
    @Override
    public String toString() {
        return this.getClass().getName() + "(tabularType=" + this.tabularType.toString() + ",contents=" + this.dataMap.toString() + ")";
    }
    
    private List<?> internalCalculateIndex(CompositeData compositeData) {
        return Collections.unmodifiableList(Arrays.asList(compositeData.getAll(this.indexNamesArray)));
    }
    
    private void checkKeyType(Object[] array) {
        if (array == null || array.length == 0) {
            throw new NullPointerException("Argument key cannot be null or empty.");
        }
        if (array.length != this.indexNamesArray.length) {
            throw new InvalidKeyException("Argument key's length=" + array.length + " is different from the number of item values, which is " + this.indexNamesArray.length + ", specified for the indexing rows in this TabularData instance.");
        }
        int i = 0;
        while (i < array.length) {
            OpenType<Object> type = this.tabularType.getRowType().getType(this.indexNamesArray[i]);
            if (array[i] != null && !type.isValue(array[i])) {
                throw new InvalidKeyException("Argument element key[" + i + "] is not a value for the open type expected for this element of the index, whose name is \"" + this.indexNamesArray[i] + "\" and whose open type is " + type);
            }
            ++i;
        }
        return;
    }
    
    private void checkValueType(CompositeData compositeData) {
        if (compositeData == null) {
            throw new NullPointerException("Argument value cannot be null.");
        }
        if (!this.tabularType.getRowType().isValue(compositeData)) {
            throw new InvalidOpenTypeException("Argument value's composite type [" + compositeData.getCompositeType() + "] is not assignable to this TabularData instance's row type [" + this.tabularType.getRowType() + "].");
        }
        return;
    }
    
    private List<?> checkValueAndIndex(CompositeData compositeData) {
        this.checkValueType(compositeData);
        List<Object> internalCalculateIndex = this.internalCalculateIndex(compositeData);
        if (this.dataMap.containsKey(internalCalculateIndex)) {
            throw new KeyAlreadyExistsException("Argument value's index, calculated according to this TabularData instance's tabularType, already refers to a value in this table.");
        }
        return internalCalculateIndex;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        List<String> indexNames = this.tabularType.getIndexNames();
        int size = indexNames.size();
        SharedSecrets.getJavaOISAccess().checkArray(objectInputStream, String[].class, size);
        this.indexNamesArray = (String[])indexNames.toArray(new String[size]);
    }
}
